@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -83,6 +83,9 @@ struct flb_task {
     const char *buf;                    /* buffer                    */
     size_t size;                        /* buffer data size          */
     void *ic;                           /* input chunk */
+#ifdef FLB_HAVE_METRICS
+    int records;                        /* numbers of records in 'buf'   */
+#endif
     struct mk_list threads;             /* ref flb_input_instance->tasks */
     struct mk_list routes;              /* routes to dispatch data       */
     struct mk_list retries;             /* queued in-memory retries      */
@@ -91,6 +94,9 @@ struct flb_task {
     struct flb_config *config;          /* parent flb config             */
 };
 
+int flb_task_running_count(struct flb_config *config);
+int flb_task_running_print(struct flb_config *config);
+
 struct flb_task *flb_task_create(uint64_t ref_id,
                                  const char *buf,
                                  size_t size,
