@@ -38,20 +38,20 @@
     "  rotated INTEGER DEFAULT 0"                                       \
     ");"
 
-#define SQL_GET_FILE "SELECT * from in_tail_files WHERE inode=%"PRIu64";"
+#define SQL_GET_FILE "SELECT * from in_tail_files WHERE inode=@inode;"
 
 #define SQL_INSERT_FILE                                             \
     "INSERT INTO in_tail_files (name, offset, inode, created)"      \
-    "  VALUES ('%s', %"PRIu64", %"PRIu64", %"PRIu64");"
+    "  VALUES (@name, @offset, @inode, @created);"
+
+#define SQL_ROTATE_FILE                                                 \
+    "UPDATE in_tail_files set name=@name,rotated=1 WHERE id=@id;"
 
 #define SQL_UPDATE_OFFSET                                   \
     "UPDATE in_tail_files set offset=@offset WHERE id=@id;"
 
-#define SQL_ROTATE_FILE                                                 \
-    "UPDATE in_tail_files set name='%s',rotated=1 WHERE id=%"PRId64";"
-
 #define SQL_DELETE_FILE                                                 \
-    "DELETE FROM in_tail_files WHERE id=%"PRId64";"
+    "DELETE FROM in_tail_files WHERE id=@id;"
 
 #define SQL_PRAGMA_SYNC                         \
     "PRAGMA synchronous=%i;"
