@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,13 +18,14 @@
  *  limitations under the License.
  */
 
-#include <fluent-bit/flb_info.h>
-#include <fluent-bit/flb_output.h>
+#include <fluent-bit/flb_output_plugin.h>
 #include <fluent-bit/flb_utils.h>
 #include <fluent-bit/flb_network.h>
 #include <fluent-bit/flb_http_client.h>
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_time.h>
+#include <fluent-bit/flb_signv4.h>
+#include <fluent-bit/flb_aws_credentials.h>
 #include <msgpack.h>
 
 #include <time.h>
@@ -36,9 +37,43 @@
 
 struct flb_output_plugin out_es_plugin;
 
-static inline int es_pack_map_content(msgpack_packer *tmp_pck,
-                                      msgpack_object map,
-                                      struct flb_elasticsearch *ctx)
+static int es_pack_array_content(msgpack_packer *tmp_pck,
+                                 msgpack_object array,
+                                 struct flb_elasticsearch *ctx);
+
+#ifdef FLB_HAVE_AWS
+static flb_sds_t add_aws_auth(struct flb_http_client *c,
+                              struct flb_elasticsearch *ctx)
+{
+    flb_sds_t signature = NULL;
+    int ret;
+
+    flb_plg_debug(ctx->ins, "Signing request with AWS Sigv4");
+
+    /* Amazon ES Sigv4 does not allow the host header to include the port */
+    ret = flb_http_strip_port_from_host(c);
+    if (ret < 0) {
+        flb_plg_error(ctx->ins, "could not strip port from host for sigv4");
+        return NULL;
+    }
+
+    /* AWS Fluent Bit user agent */
+    flb_http_add_header(c, "User-Agent", 10, "aws-fluent-bit-plugin", 21);
+
+    signature = flb_signv4_do(c, FLB_TRUE, FLB_TRUE, time(NULL),
+                              ctx->aws_region, "es",
+                              ctx->aws_provider);
+    if (!signature) {
+        flb_plg_error(ctx->ins, "could not sign request with sigv4");
+        return NULL;
+    }
+    return signature;
+}
+#endif /* FLB_HAVE_AWS */
+
+static int es_pack_map_content(msgpack_packer *tmp_pck,
+                               msgpack_object map,
+                               struct flb_elasticsearch *ctx)
 {
     int i;
     char *ptr_key = NULL;
@@ -114,11 +149,49 @@ static inline int es_pack_map_content(msgpack_packer *tmp_pck,
             msgpack_pack_map(tmp_pck, v->via.map.size);
             es_pack_map_content(tmp_pck, *v, ctx);
         }
+        /*
+         * The value can be any data type, if it's an array we need to
+         * pass it to es_pack_array_content.
+         */
+        else if (v->type == MSGPACK_OBJECT_ARRAY) {
+          msgpack_pack_array(tmp_pck, v->via.array.size);
+          es_pack_array_content(tmp_pck, *v, ctx);
+        }
         else {
             msgpack_pack_object(tmp_pck, *v);
         }
     }
+    return 0;
+}
 
+/*
+  * Iterate through the array and sanitize elements.
+  * Mutual recursion with es_pack_map_content.
+  */
+static int es_pack_array_content(msgpack_packer *tmp_pck,
+                                 msgpack_object array,
+                                 struct flb_elasticsearch *ctx)
+{
+    int i;
+    msgpack_object *e;
+
+    for (i = 0; i < array.via.array.size; i++) {
+        e = &array.via.array.ptr[i];
+        if (e->type == MSGPACK_OBJECT_MAP)
+        {
+            msgpack_pack_map(tmp_pck, e->via.map.size);
+            es_pack_map_content(tmp_pck, *e, ctx);
+        }
+        else if (e->type == MSGPACK_OBJECT_ARRAY)
+        {
+            msgpack_pack_array(tmp_pck, e->via.array.size);
+            es_pack_array_content(tmp_pck, *e, ctx);
+        }
+        else
+        {
+            msgpack_pack_object(tmp_pck, *e);
+        }
+    }
     return 0;
 }
 
@@ -128,9 +201,13 @@ static inline int es_pack_map_content(msgpack_packer *tmp_pck,
  *
  * 'Sadly' this process involves to convert from Msgpack to JSON.
  */
-static char *elasticsearch_format(const void *data, size_t bytes,
-                                  const char *tag, int tag_len, int *out_size,
-                                  struct flb_elasticsearch *ctx)
+static int elasticsearch_format(struct flb_config *config,
+                                struct flb_input_instance *ins,
+                                void *plugin_context,
+                                void *flush_ctx,
+                                const char *tag, int tag_len,
+                                const void *data, size_t bytes,
+                                void **out_data, size_t *out_size)
 {
     int ret;
     int len;
@@ -139,7 +216,6 @@ static char *elasticsearch_format(const void *data, size_t bytes,
     size_t s = 0;
     size_t off = 0;
     char *p;
-    char *buf;
     char *es_index;
     char logstash_index[256];
     char time_formatted[256];
@@ -162,6 +238,7 @@ static char *elasticsearch_format(const void *data, size_t bytes,
     int i;
     msgpack_object key;
     msgpack_object val;
+    struct flb_elasticsearch *ctx = plugin_context;
 
     /* Iterate the original buffer and perform adjustments */
     msgpack_unpacked_init(&result);
@@ -170,7 +247,7 @@ static char *elasticsearch_format(const void *data, size_t bytes,
     ret = msgpack_unpack_next(&result, data, bytes, &off);
     if (ret != MSGPACK_UNPACK_SUCCESS) {
         msgpack_unpacked_destroy(&result);
-        return NULL;
+        return -1;
     }
 
     /* We 'should' get an array */
@@ -180,18 +257,18 @@ static char *elasticsearch_format(const void *data, size_t bytes,
          * doing, we just duplicate the content in a new buffer and cleanup.
          */
         msgpack_unpacked_destroy(&result);
-        return NULL;
+        return -1;
     }
 
     root = result.data;
     if (root.via.array.size == 0) {
-        return NULL;
+        return -1;
     }
 
     /* Create the bulk composer */
     bulk = es_bulk_create();
     if (!bulk) {
-        return NULL;
+        return -1;
     }
 
     off = 0;
@@ -201,8 +278,8 @@ static char *elasticsearch_format(const void *data, size_t bytes,
 
     /* Copy logstash prefix if logstash format is enabled */
     if (ctx->logstash_format == FLB_TRUE) {
-        memcpy(logstash_index, ctx->logstash_prefix, ctx->logstash_prefix_len);
-        logstash_index[ctx->logstash_prefix_len] = '\0';
+        memcpy(logstash_index, ctx->logstash_prefix, flb_sds_len(ctx->logstash_prefix));
+        logstash_index[flb_sds_len(ctx->logstash_prefix)] = '\0';
     }
 
     /*
@@ -263,16 +340,17 @@ static char *elasticsearch_format(const void *data, size_t bytes,
         map_size = map.via.map.size;
 
         es_index_custom_len = 0;
-        if (ctx->logstash_prefix_key_len != 0) {
+        if (ctx->logstash_prefix_key) {
             for (i = 0; i < map_size; i++) {
                 key = map.via.map.ptr[i].key;
                 if (key.type != MSGPACK_OBJECT_STR) {
                     continue;
                 }
-                if (key.via.str.size != ctx->logstash_prefix_key_len) {
+                if (key.via.str.size != flb_sds_len(ctx->logstash_prefix_key)) {
                     continue;
                 }
-                if (strncmp(key.via.str.ptr, ctx->logstash_prefix_key, ctx->logstash_prefix_key_len) != 0) {
+                if (strncmp(key.via.str.ptr, ctx->logstash_prefix_key,
+                            flb_sds_len(ctx->logstash_prefix_key)) != 0) {
                     continue;
                 }
                 val = map.via.map.ptr[i].val;
@@ -302,8 +380,8 @@ static char *elasticsearch_format(const void *data, size_t bytes,
         msgpack_pack_map(&tmp_pck, map_size + 1);
 
         /* Append the time key */
-        msgpack_pack_str(&tmp_pck, ctx->time_key_len);
-        msgpack_pack_str_body(&tmp_pck, ctx->time_key, ctx->time_key_len);
+        msgpack_pack_str(&tmp_pck, flb_sds_len(ctx->time_key));
+        msgpack_pack_str_body(&tmp_pck, ctx->time_key, flb_sds_len(ctx->time_key));
 
         /* Format the time */
         gmtime_r(&tms.tm.tv_sec, &tm);
@@ -322,7 +400,7 @@ static char *elasticsearch_format(const void *data, size_t bytes,
             if (es_index_custom_len > 0) {
                 p = logstash_index + es_index_custom_len;
             } else {
-                p = logstash_index + ctx->logstash_prefix_len;
+                p = logstash_index + flb_sds_len(ctx->logstash_prefix);
             }
             *p++ = '-';
 
@@ -348,8 +426,8 @@ static char *elasticsearch_format(const void *data, size_t bytes,
 
         /* Tag Key */
         if (ctx->include_tag_key == FLB_TRUE) {
-            msgpack_pack_str(&tmp_pck, ctx->tag_key_len);
-            msgpack_pack_str_body(&tmp_pck, ctx->tag_key, ctx->tag_key_len);
+            msgpack_pack_str(&tmp_pck, flb_sds_len(ctx->tag_key));
+            msgpack_pack_str_body(&tmp_pck, ctx->tag_key, flb_sds_len(ctx->tag_key));
             msgpack_pack_str(&tmp_pck, tag_len);
             msgpack_pack_str_body(&tmp_pck, tag, tag_len);
         }
@@ -366,7 +444,7 @@ static char *elasticsearch_format(const void *data, size_t bytes,
             msgpack_unpacked_destroy(&result);
             msgpack_sbuffer_destroy(&tmp_sbuf);
             es_bulk_destroy(bulk);
-            return NULL;
+            return -1;
         }
 
         if (ctx->generate_id == FLB_TRUE) {
@@ -387,7 +465,7 @@ static char *elasticsearch_format(const void *data, size_t bytes,
         if (!out_buf) {
             msgpack_unpacked_destroy(&result);
             es_bulk_destroy(bulk);
-            return NULL;
+            return -1;
         }
 
         ret = es_bulk_append(bulk, j_index, index_len,
@@ -398,13 +476,14 @@ static char *elasticsearch_format(const void *data, size_t bytes,
             msgpack_unpacked_destroy(&result);
             *out_size = 0;
             es_bulk_destroy(bulk);
-            return NULL;
+            return -1;
         }
     }
     msgpack_unpacked_destroy(&result);
 
+    /* Set outgoing data */
+    *out_data = bulk->ptr;
     *out_size = bulk->len;
-    buf = bulk->ptr;
 
     /*
      * Note: we don't destroy the bulk as we need to keep the allocated
@@ -413,10 +492,11 @@ static char *elasticsearch_format(const void *data, size_t bytes,
      */
     flb_free(bulk);
     if (ctx->trace_output) {
-        fwrite(buf, 1, *out_size, stdout);
+        fwrite(*out_data, 1, *out_size, stdout);
         fflush(stdout);
     }
-    return buf;
+
+    return 0;
 }
 
 static int cb_es_init(struct flb_output_instance *ins,
@@ -427,19 +507,27 @@ static int cb_es_init(struct flb_output_instance *ins,
 
     ctx = flb_es_conf_create(ins, config);
     if (!ctx) {
-        flb_error("[out_es] cannot initialize plugin");
+        flb_plg_error(ins, "cannot initialize plugin");
         return -1;
     }
 
-    flb_debug("[out_es] host=%s port=%i uri=%s index=%s type=%s",
-              ins->host.name, ins->host.port, ctx->uri,
-              ctx->index, ctx->type);
+    flb_plg_debug(ctx->ins, "host=%s port=%i uri=%s index=%s type=%s",
+                  ins->host.name, ins->host.port, ctx->uri,
+                  ctx->index, ctx->type);
 
     flb_output_set_context(ins, ctx);
+
+    /*
+     * This plugin instance uses the HTTP client interface, let's register
+     * it debugging callbacks.
+     */
+    flb_output_set_http_debug_callbacks(ins);
+
     return 0;
 }
 
-static int elasticsearch_error_check(struct flb_http_client *c)
+static int elasticsearch_error_check(struct flb_elasticsearch *ctx,
+                                     struct flb_http_client *c)
 {
     int i;
     int ret;
@@ -472,8 +560,8 @@ static int elasticsearch_error_check(struct flb_http_client *c)
             return FLB_FALSE;
         }
 
-        flb_error("[out_es] could not pack/validate JSON response\n%s",
-                  c->resp.payload);
+        flb_plg_error(ctx->ins, "could not pack/validate JSON response\n%s",
+                      c->resp.payload);
         return FLB_TRUE;
     }
 
@@ -481,15 +569,15 @@ static int elasticsearch_error_check(struct flb_http_client *c)
     msgpack_unpacked_init(&result);
     ret = msgpack_unpack_next(&result, out_buf, out_size, &off);
     if (ret != MSGPACK_UNPACK_SUCCESS) {
-        flb_error("[out_es] Cannot unpack response to find error\n%s",
-                  c->resp.payload);
+        flb_plg_error(ctx->ins, "Cannot unpack response to find error\n%s",
+                      c->resp.payload);
         return FLB_TRUE;
     }
 
     root = result.data;
     if (root.type != MSGPACK_OBJECT_MAP) {
-        flb_error("[out_es] unexpected payload type=%i",
-                  root.type);
+        flb_plg_error(ctx->ins, "unexpected payload type=%i",
+                      root.type);
         check = FLB_TRUE;
         goto done;
     }
@@ -497,8 +585,8 @@ static int elasticsearch_error_check(struct flb_http_client *c)
     for (i = 0; i < root.via.map.size; i++) {
         key = root.via.map.ptr[i].key;
         if (key.type != MSGPACK_OBJECT_STR) {
-            flb_error("[out_es] unexpected key type=%i",
-                      key.type);
+            flb_plg_error(ctx->ins, "unexpected key type=%i",
+                          key.type);
             check = FLB_TRUE;
             goto done;
         }
@@ -510,8 +598,8 @@ static int elasticsearch_error_check(struct flb_http_client *c)
         if (strncmp(key.via.str.ptr, "errors", 6) == 0) {
             val = root.via.map.ptr[i].val;
             if (val.type != MSGPACK_OBJECT_BOOLEAN) {
-                flb_error("[out_es] unexpected 'error' value type=%i",
-                          val.type);
+                flb_plg_error(ctx->ins, "unexpected 'error' value type=%i",
+                              val.type);
                 check = FLB_TRUE;
                 goto done;
             }
@@ -536,21 +624,21 @@ static int elasticsearch_error_check(struct flb_http_client *c)
     return check;
 }
 
-void cb_es_flush(const void *data, size_t bytes,
-                 const char *tag, int tag_len,
-                 struct flb_input_instance *i_ins, void *out_context,
-                 struct flb_config *config)
+static void cb_es_flush(const void *data, size_t bytes,
+                        const char *tag, int tag_len,
+                        struct flb_input_instance *ins, void *out_context,
+                        struct flb_config *config)
 {
     int ret;
-    int bytes_out;
+    size_t pack_size;
     char *pack;
+    void *out_buf;
+    size_t out_size;
     size_t b_sent;
     struct flb_elasticsearch *ctx = out_context;
     struct flb_upstream_conn *u_conn;
     struct flb_http_client *c;
-    (void) i_ins;
-    (void) tag;
-    (void) tag_len;
+    flb_sds_t signature = NULL;
 
     /* Get upstream connection */
     u_conn = flb_upstream_conn_get(ctx->u);
@@ -559,34 +647,64 @@ void cb_es_flush(const void *data, size_t bytes,
     }
 
     /* Convert format */
-    pack = elasticsearch_format(data, bytes, tag, tag_len, &bytes_out, ctx);
-    if (!pack) {
+    ret = elasticsearch_format(config, ins,
+                               ctx, NULL,
+                               tag, tag_len,
+                               data, bytes,
+                               &out_buf, &out_size);
+    if (ret != 0) {
         flb_upstream_conn_release(u_conn);
         FLB_OUTPUT_RETURN(FLB_ERROR);
     }
 
+    pack = (char *) out_buf;
+    pack_size = out_size;
+
     /* Compose HTTP Client request */
     c = flb_http_client(u_conn, FLB_HTTP_POST, ctx->uri,
-                        pack, bytes_out, NULL, 0, NULL, 0);
+                        pack, pack_size, NULL, 0, NULL, 0);
 
     flb_http_buffer_size(c, ctx->buffer_size);
 
+#ifndef FLB_HAVE_AWS
     flb_http_add_header(c, "User-Agent", 10, "Fluent-Bit", 10);
+#endif
+
     flb_http_add_header(c, "Content-Type", 12, "application/x-ndjson", 20);
 
     if (ctx->http_user && ctx->http_passwd) {
         flb_http_basic_auth(c, ctx->http_user, ctx->http_passwd);
     }
 
+#ifdef FLB_HAVE_AWS
+    if (ctx->has_aws_auth == FLB_TRUE) {
+        signature = add_aws_auth(c, ctx);
+        if (!signature) {
+            goto retry;
+        }
+    }
+    else {
+        flb_http_add_header(c, "User-Agent", 10, "Fluent-Bit", 10);
+    }
+#endif
+
     ret = flb_http_do(c, &b_sent);
     if (ret != 0) {
-        flb_warn("[out_es] http_do=%i URI=%s", ret, ctx->uri);
+        flb_plg_warn(ctx->ins, "http_do=%i URI=%s", ret, ctx->uri);
         goto retry;
     }
     else {
         /* The request was issued successfully, validate the 'error' field */
-        flb_debug("[out_es] HTTP Status=%i URI=%s", c->resp.status, ctx->uri);
+        flb_plg_debug(ctx->ins, "HTTP Status=%i URI=%s", c->resp.status, ctx->uri);
         if (c->resp.status != 200 && c->resp.status != 201) {
+            if (c->resp.payload_size > 0) {
+                flb_plg_error(ctx->ins, "HTTP status=%i URI=%s, response:\n%s\n",
+                              c->resp.status, ctx->uri, c->resp.payload);
+            }
+            else {
+                flb_plg_error(ctx->ins, "HTTP status=%i URI=%s",
+                              c->resp.status, ctx->uri);
+            }
             goto retry;
         }
 
@@ -595,7 +713,7 @@ void cb_es_flush(const void *data, size_t bytes,
              * Elasticsearch payload should be JSON, we convert it to msgpack
              * and lookup the 'error' field.
              */
-            ret = elasticsearch_error_check(c);
+            ret = elasticsearch_error_check(ctx, c);
             if (ret == FLB_TRUE) {
                 /* we got an error */
                 if (ctx->trace_error) {
@@ -603,14 +721,16 @@ void cb_es_flush(const void *data, size_t bytes,
                      * If trace_error is set, trace the actual
                      * input/output to Elasticsearch that caused the problem.
                      */
-                    flb_error("[out_es] error: Input\n%s\nOutput\n%s",
-                             pack, c->resp.payload);
+                    flb_plg_debug(ctx->ins, "error caused by: Input\n%s\n",
+                                   pack);
+                    flb_plg_error(ctx->ins, "error: Output\n%s",
+                                  c->resp.payload);
                 }
                 goto retry;
             }
             else {
-                flb_debug("[out_es Elasticsearch response\n%s",
-                          c->resp.payload);
+                flb_plg_debug(ctx->ins, "Elasticsearch response\n%s",
+                              c->resp.payload);
             }
         }
         else {
@@ -622,6 +742,9 @@ void cb_es_flush(const void *data, size_t bytes,
     flb_http_client_destroy(c);
     flb_free(pack);
     flb_upstream_conn_release(u_conn);
+    if (signature) {
+        flb_sds_destroy(signature);
+    }
     FLB_OUTPUT_RETURN(FLB_OK);
 
     /* Issue a retry */
@@ -632,7 +755,7 @@ void cb_es_flush(const void *data, size_t bytes,
     FLB_OUTPUT_RETURN(FLB_RETRY);
 }
 
-int cb_es_exit(void *data, struct flb_config *config)
+static int cb_es_exit(void *data, struct flb_config *config)
 {
     struct flb_elasticsearch *ctx = data;
 
@@ -640,6 +763,148 @@ int cb_es_exit(void *data, struct flb_config *config)
     return 0;
 }
 
+/* Configuration properties map */
+static struct flb_config_map config_map[] = {
+    {
+     FLB_CONFIG_MAP_STR, "index", FLB_ES_DEFAULT_INDEX,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, index),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_STR, "type", FLB_ES_DEFAULT_TYPE,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, type),
+     NULL
+    },
+
+    /* HTTP Authentication */
+    {
+     FLB_CONFIG_MAP_STR, "http_user", NULL,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, http_user),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_STR, "http_passwd", "",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, http_passwd),
+     NULL
+    },
+
+    /* AWS Authentication */
+#ifdef FLB_HAVE_AWS
+    {
+     FLB_CONFIG_MAP_BOOL, "aws_auth", "false",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, has_aws_auth),
+     "Enable AWS Sigv4 Authentication"
+    },
+    {
+     FLB_CONFIG_MAP_STR, "aws_region", "",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, aws_region),
+     "AWS Region of your Amazon ElasticSearch Service cluster"
+    },
+    {
+     FLB_CONFIG_MAP_STR, "aws_role_arn", "",
+     0, FLB_FALSE, 0,
+     "AWS IAM Role to assume to put records to your Amazon ES cluster"
+    },
+    {
+     FLB_CONFIG_MAP_STR, "aws_external_id", "",
+     0, FLB_FALSE, 0,
+     "External ID for the AWS IAM Role specified with `aws_role_arn`"
+    },
+#endif
+
+    /* Logstash compatibility */
+    {
+     FLB_CONFIG_MAP_BOOL, "logstash_format", "false",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, logstash_format),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_STR, "logstash_prefix", FLB_ES_DEFAULT_PREFIX,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, logstash_prefix),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_STR, "logstash_prefix_key", NULL,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, logstash_prefix_key),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_STR, "logstash_dateformat", FLB_ES_DEFAULT_TIME_FMT,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, logstash_dateformat),
+     NULL
+    },
+
+    /* Custom Time and Tag keys */
+    {
+     FLB_CONFIG_MAP_STR, "time_key", FLB_ES_DEFAULT_TIME_KEY,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, time_key),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_STR, "time_key_format", FLB_ES_DEFAULT_TIME_KEYF,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, time_key_format),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_BOOL, "include_tag_key", "false",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, include_tag_key),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_STR, "tag_key", FLB_ES_DEFAULT_TAG_KEY,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, tag_key),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_SIZE, "buffer_size", FLB_ES_DEFAULT_HTTP_MAX,
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, buffer_size),
+     NULL
+    },
+
+    /* Elasticsearch specifics */
+    {
+     FLB_CONFIG_MAP_STR, "path", NULL,
+     0, FLB_FALSE, 0,
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_STR, "pipeline", NULL,
+     0, FLB_FALSE, 0,
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_BOOL, "generate_id", "false",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, generate_id),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_BOOL, "replace_dots", "false",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, replace_dots),
+     NULL
+    },
+
+    {
+     FLB_CONFIG_MAP_BOOL, "current_time_index", "false",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, current_time_index),
+     NULL
+    },
+
+    /* Trace */
+    {
+     FLB_CONFIG_MAP_BOOL, "trace_output", "false",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, trace_output),
+     NULL
+    },
+    {
+     FLB_CONFIG_MAP_BOOL, "trace_error", "false",
+     0, FLB_TRUE, offsetof(struct flb_elasticsearch, trace_error),
+     NULL
+    },
+
+    /* EOF */
+    {0}
+};
+
 /* Plugin reference */
 struct flb_output_plugin out_es_plugin = {
     .name           = "es",
@@ -649,6 +914,12 @@ struct flb_output_plugin out_es_plugin = {
     .cb_flush       = cb_es_flush,
     .cb_exit        = cb_es_exit,
 
+    /* Configuration */
+    .config_map     = config_map,
+
+    /* Test */
+    .test_formatter.callback = elasticsearch_format,
+
     /* Plugin flags */
     .flags          = FLB_OUTPUT_NET | FLB_IO_OPT_TLS,
 };
