@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -35,6 +35,8 @@
 #include <fluent-bit/flb_storage.h>
 #include <fluent-bit/flb_kv.h>
 
+struct flb_libco_in_params libco_in_param;
+
 #define protcmp(a, b)  strncasecmp(a, b, strlen(a))
 
 static int check_protocol(const char *prot, const char *output)
@@ -42,7 +44,7 @@ static int check_protocol(const char *prot, const char *output)
     int len;
 
     len = strlen(prot);
-    if (len > strlen(output)) {
+    if (len != strlen(output)) {
         return 0;
     }
 
@@ -71,21 +73,33 @@ static inline int instance_id(struct flb_input_plugin *p,
 }
 
 /* Generate a new collector ID for the instance in question */
-static int collector_id(struct flb_input_instance *in)
+static int collector_id(struct flb_input_instance *ins)
 {
     int id = 0;
     struct flb_input_collector *collector;
 
-    if (mk_list_is_empty(&in->collectors) == 0) {
+    if (mk_list_is_empty(&ins->collectors) == 0) {
         return id;
     }
 
-    collector = mk_list_entry_last(&in->collectors,
+    collector = mk_list_entry_last(&ins->collectors,
                                    struct flb_input_collector,
                                    _head_ins);
     return (collector->id + 1);
 }
 
+void flb_input_net_default_listener(const char *listen, int port,
+                                    struct flb_input_instance *ins)
+{
+    /* Set default network configuration */
+    if (!ins->host.listen) {
+        ins->host.listen = flb_sds_create(listen);
+    }
+    if (ins->host.port == 0) {
+        ins->host.port = port;
+    }
+}
+
 /* Create an input plugin instance */
 struct flb_input_instance *flb_input_new(struct flb_config *config,
                                          const char *input, void *data,
@@ -142,11 +156,14 @@ struct flb_input_instance *flb_input_new(struct flb_config *config,
         instance->data     = data;
         instance->threaded = FLB_FALSE;
         instance->storage  = NULL;
+        instance->storage_type = -1;
+        instance->log_level = -1;
 
         /* net */
         instance->host.name    = NULL;
         instance->host.address = NULL;
         instance->host.uri     = NULL;
+        instance->host.listen  = NULL;
         instance->host.ipv6    = FLB_FALSE;
 
         /* Initialize list heads */
@@ -197,17 +214,39 @@ static inline int prop_key_check(const char *key, const char *kv, int k_len)
     return -1;
 }
 
+int flb_input_name_exists(const char *name, struct flb_config *config)
+{
+    struct mk_list *head;
+    struct flb_input_instance *ins;
+
+    mk_list_foreach(head, &config->inputs) {
+        ins = mk_list_entry(head, struct flb_input_instance, _head);
+        if (strcmp(ins->name, name) == 0) {
+            return FLB_TRUE;
+        }
+
+        if (ins->alias) {
+            if (strcmp(ins->alias, name) == 0) {
+                return FLB_TRUE;
+            }
+        }
+    }
+
+    return FLB_FALSE;
+}
+
 /* Override a configuration property for the given input_instance plugin */
-int flb_input_set_property(struct flb_input_instance *in,
+int flb_input_set_property(struct flb_input_instance *ins,
                            const char *k, const char *v)
 {
     int len;
+    int ret;
     ssize_t limit;
     flb_sds_t tmp = NULL;
     struct flb_kv *kv;
 
     len = strlen(k);
-    tmp = flb_env_var_translate(in->config->env, v);
+    tmp = flb_env_var_translate(ins->config->env, v);
     if (tmp) {
         if (flb_sds_len(tmp) == 0) {
             flb_sds_destroy(tmp);
@@ -217,15 +256,23 @@ int flb_input_set_property(struct flb_input_instance *in,
 
     /* Check if the key is a known/shared property */
     if (prop_key_check("tag", k, len) == 0 && tmp) {
-        in->tag     = tmp;
-        in->tag_len = flb_sds_len(tmp);
+        ins->tag     = tmp;
+        ins->tag_len = flb_sds_len(tmp);
+    }
+    else if (prop_key_check("log_level", k, len) == 0 && tmp) {
+        ret = flb_log_get_level_str(tmp);
+        flb_sds_destroy(tmp);
+        if (ret == -1) {
+            return -1;
+        }
+        ins->log_level = ret;
     }
     else if (prop_key_check("routable", k, len) == 0 && tmp) {
-        in->routable = flb_utils_bool(tmp);
+        ins->routable = flb_utils_bool(tmp);
         flb_sds_destroy(tmp);
     }
     else if (prop_key_check("alias", k, len) == 0 && tmp) {
-        in->alias = tmp;
+        ins->alias = tmp;
     }
     else if (prop_key_check("mem_buf_limit", k, len) == 0 && tmp) {
         limit = flb_utils_size_to_bytes(tmp);
@@ -233,22 +280,36 @@ int flb_input_set_property(struct flb_input_instance *in,
         if (limit == -1) {
             return -1;
         }
-        in->mem_buf_limit = (size_t) limit;
+        ins->mem_buf_limit = (size_t) limit;
     }
     else if (prop_key_check("listen", k, len) == 0) {
-        in->host.listen = tmp;
+        ins->host.listen = tmp;
     }
     else if (prop_key_check("host", k, len) == 0) {
-        in->host.name   = tmp;
+        ins->host.name   = tmp;
     }
     else if (prop_key_check("port", k, len) == 0) {
         if (tmp) {
-            in->host.port = atoi(tmp);
+            ins->host.port = atoi(tmp);
             flb_sds_destroy(tmp);
         }
     }
     else if (prop_key_check("ipv6", k, len) == 0 && tmp) {
-        in->host.ipv6 = flb_utils_bool(tmp);
+        ins->host.ipv6 = flb_utils_bool(tmp);
+        flb_sds_destroy(tmp);
+    }
+    else if (prop_key_check("storage.type", k, len) == 0 && tmp) {
+        /* Set the storage type */
+        if (strcasecmp(tmp, "filesystem") == 0) {
+            ins->storage_type = CIO_STORE_FS;
+        }
+        else if (strcasecmp(tmp, "memory") == 0) {
+            ins->storage_type = CIO_STORE_MEM;
+        }
+        else {
+            flb_sds_destroy(tmp);
+            return -1;
+        }
         flb_sds_destroy(tmp);
     }
     else {
@@ -256,7 +317,7 @@ int flb_input_set_property(struct flb_input_instance *in,
          * Create the property, we don't pass the value since we will
          * map it directly to avoid an extra memory allocation.
          */
-        kv = flb_kv_item_create(&in->properties, (char *) k, NULL);
+        kv = flb_kv_item_create(&ins->properties, (char *) k, NULL);
         if (!kv) {
             if (tmp) {
                 flb_sds_destroy(tmp);
@@ -269,66 +330,86 @@ int flb_input_set_property(struct flb_input_instance *in,
     return 0;
 }
 
-const char *flb_input_get_property(const char *key, struct flb_input_instance *i)
+const char *flb_input_get_property(const char *key,
+                                   struct flb_input_instance *ins)
 {
-    return flb_config_prop_get(key, &i->properties);
+    return flb_config_prop_get(key, &ins->properties);
 }
 
 /* Return an instance name or alias */
-const char *flb_input_name(struct flb_input_instance *in)
+const char *flb_input_name(struct flb_input_instance *ins)
 {
-    if (in->alias) {
-        return in->alias;
+    if (ins->alias) {
+        return ins->alias;
     }
 
-    return in->name;
+    return ins->name;
 }
 
-void flb_input_instance_free(struct flb_input_instance *in)
+void flb_input_instance_destroy(struct flb_input_instance *ins)
 {
-    if (in->alias) {
-        flb_sds_destroy(in->alias);
+    if (ins->alias) {
+        flb_sds_destroy(ins->alias);
     }
 
     /* Remove URI context */
-    if (in->host.uri) {
-        flb_uri_destroy(in->host.uri);
+    if (ins->host.uri) {
+        flb_uri_destroy(ins->host.uri);
     }
 
-    flb_sds_destroy(in->host.name);
-    flb_sds_destroy(in->host.address);
+    if (ins->host.name) {
+        flb_sds_destroy(ins->host.name);
+    }
+    if (ins->host.address) {
+        flb_sds_destroy(ins->host.address);
+    }
+    if (ins->host.listen) {
+        flb_sds_destroy(ins->host.listen);
+    }
 
     /* release the tag if any */
-    flb_sds_destroy(in->tag);
+    flb_sds_destroy(ins->tag);
 
     /* Let the engine remove any pending task */
-    flb_engine_destroy_tasks(&in->tasks);
+    flb_engine_destroy_tasks(&ins->tasks);
 
     /* release properties */
-    flb_kv_release(&in->properties);
+    flb_kv_release(&ins->properties);
 
     /* Remove metrics */
 #ifdef FLB_HAVE_METRICS
-    if (in->metrics) {
-        flb_metrics_destroy(in->metrics);
+    if (ins->metrics) {
+        flb_metrics_destroy(ins->metrics);
     }
 #endif
 
-    if (in->storage) {
-        flb_storage_input_destroy(in);
+    if (ins->storage) {
+        flb_storage_input_destroy(ins);
+    }
+
+    /* destroy config map */
+    if (ins->config_map) {
+        flb_config_map_destroy(ins->config_map);
     }
 
     /* Unlink and release */
-    mk_list_del(&in->_head);
-    flb_free(in);
+    mk_list_del(&ins->_head);
+    flb_free(ins);
 }
 
-int flb_input_instance_init(struct flb_input_instance *in,
+int flb_input_instance_init(struct flb_input_instance *ins,
                             struct flb_config *config)
 {
     int ret;
+#ifdef FLB_HAVE_METRICS
     const char *name;
-    struct flb_input_plugin *p = in->p;
+#endif
+    struct mk_list *config_map;
+    struct flb_input_plugin *p = ins->p;
+
+    if (ins->log_level == -1) {
+        ins->log_level = config->log->level;
+    }
 
     /* Skip pseudo input plugins */
     if (!p) {
@@ -338,28 +419,58 @@ int flb_input_instance_init(struct flb_input_instance *in,
     /* Metrics */
 #ifdef FLB_HAVE_METRICS
     /* Get name or alias for the instance */
-    name = flb_input_name(in);
+    name = flb_input_name(ins);
 
     /* Create the metrics context */
-    in->metrics = flb_metrics_create(name);
-    if (in->metrics) {
-        flb_metrics_add(FLB_METRIC_N_RECORDS, "records", in->metrics);
-        flb_metrics_add(FLB_METRIC_N_BYTES, "bytes", in->metrics);
+    ins->metrics = flb_metrics_create(name);
+    if (ins->metrics) {
+        flb_metrics_add(FLB_METRIC_N_RECORDS, "records", ins->metrics);
+        flb_metrics_add(FLB_METRIC_N_BYTES, "bytes", ins->metrics);
     }
 #endif
 
+    /*
+     * Before to call the initialization callback, make sure that the received
+     * configuration parameters are valid if the plugin is registering a config map.
+     */
+    if (p->config_map) {
+        /*
+         * Create a dynamic version of the configmap that will be used by the specific
+         * instance in question.
+         */
+        config_map = flb_config_map_create(config, p->config_map);
+        if (!config_map) {
+            flb_error("[filter] error loading config map for '%s' plugin",
+                      p->name);
+            return -1;
+        }
+        ins->config_map = config_map;
+
+        /* Validate incoming properties against config map */
+        ret = flb_config_map_properties_check(ins->p->name,
+                                              &ins->properties, ins->config_map);
+        if (ret == -1) {
+            if (config->program_name) {
+                flb_helper("try the command: %s -i %s -h\n",
+                           config->program_name, ins->p->name);
+            }
+            flb_input_instance_destroy(ins);
+            return -1;
+        }
+    }
+
     /* Initialize the input */
     if (p->cb_init) {
         /* Sanity check: all non-dynamic tag input plugins must have a tag */
-        if (!in->tag) {
-            flb_input_set_property(in, "tag", in->name);
+        if (!ins->tag) {
+            flb_input_set_property(ins, "tag", ins->name);
         }
 
-        ret = p->cb_init(in, config, in->data);
+        ret = p->cb_init(ins, config, ins->data);
         if (ret != 0) {
             flb_error("Failed initialize input %s",
-                      in->name);
-            flb_input_instance_free(in);
+                      ins->name);
+            flb_input_instance_destroy(ins);
             return -1;
         }
     }
@@ -369,12 +480,12 @@ int flb_input_instance_init(struct flb_input_instance *in,
 
 
 /* Initialize all inputs */
-void flb_input_initialize_all(struct flb_config *config)
+int flb_input_init_all(struct flb_config *config)
 {
     int ret;
     struct mk_list *tmp;
     struct mk_list *head;
-    struct flb_input_instance *in;
+    struct flb_input_instance *ins;
     struct flb_input_plugin *p;
 
     /* Initialize thread-id table */
@@ -382,8 +493,8 @@ void flb_input_initialize_all(struct flb_config *config)
 
     /* Iterate all active input instance plugins */
     mk_list_foreach_safe(head, tmp, &config->inputs) {
-        in = mk_list_entry(head, struct flb_input_instance, _head);
-        p = in->p;
+        ins = mk_list_entry(head, struct flb_input_instance, _head);
+        p = ins->p;
 
         /* Skip pseudo input plugins */
         if (!p) {
@@ -391,41 +502,44 @@ void flb_input_initialize_all(struct flb_config *config)
         }
 
         /* Initialize instance */
-        ret = flb_input_instance_init(in, config);
+        ret = flb_input_instance_init(ins, config);
         if (ret == -1) {
             /* do nothing, it's ok if it fails */
+            return -1;
         }
     }
+
+    return 0;
 }
 
 /* Invoke all pre-run input callbacks */
 void flb_input_pre_run_all(struct flb_config *config)
 {
     struct mk_list *head;
-    struct flb_input_instance *in;
+    struct flb_input_instance *ins;
     struct flb_input_plugin *p;
 
     mk_list_foreach(head, &config->inputs) {
-        in = mk_list_entry(head, struct flb_input_instance, _head);
-        p = in->p;
+        ins = mk_list_entry(head, struct flb_input_instance, _head);
+        p = ins->p;
         if (!p) {
             continue;
         }
 
         if (p->cb_pre_run) {
-            p->cb_pre_run(in, config, in->context);
+            p->cb_pre_run(ins, config, ins->context);
         }
     }
 }
 
-void flb_input_instance_exit(struct flb_input_instance *in,
+void flb_input_instance_exit(struct flb_input_instance *ins,
                              struct flb_config *config)
 {
     struct flb_input_plugin *p;
 
-    p = in->p;
-    if (p->cb_exit && in->context) {
-        p->cb_exit(in->context, config);
+    p = ins->p;
+    if (p->cb_exit && ins->context) {
+        p->cb_exit(ins->context, config);
     }
 }
 
@@ -434,19 +548,19 @@ void flb_input_exit_all(struct flb_config *config)
 {
     struct mk_list *tmp;
     struct mk_list *head;
-    struct flb_input_instance *in;
+    struct flb_input_instance *ins;
     struct flb_input_plugin *p;
 
     /* Iterate instances */
     mk_list_foreach_safe_r(head, tmp, &config->inputs) {
-        in = mk_list_entry(head, struct flb_input_instance, _head);
-        p = in->p;
+        ins = mk_list_entry(head, struct flb_input_instance, _head);
+        p = ins->p;
         if (!p) {
             continue;
         }
 
-        flb_input_instance_exit(in, config);
-        flb_input_instance_free(in);
+        flb_input_instance_exit(ins, config);
+        flb_input_instance_destroy(ins);
     }
 }
 
@@ -463,7 +577,7 @@ int flb_input_check(struct flb_config *config)
 /*
  * API for Input plugins
  * =====================
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  * The Input interface provides a certain number of functions that can be
  * used by Input plugins to configure it own behavior and request specific
  *
@@ -630,7 +744,7 @@ int flb_input_collector_start(int coll_id, struct flb_input_instance *in)
             ret = collector_start(coll, in->config);
             if (ret == -1) {
                 flb_error("[input] error starting collector #%i: %s",
-                          in->name);
+                          coll_id, in->name);
             }
             return ret;
         }
@@ -690,9 +804,9 @@ int flb_input_pause_all(struct flb_config *config)
 
     mk_list_foreach(head, &config->inputs) {
         in = mk_list_entry(head, struct flb_input_instance, _head);
-        flb_info("[input] pausing %s", in->name);
         if (flb_input_buf_paused(in) == FLB_FALSE) {
-            if (in->p->cb_pause) {
+            if (in->p->cb_pause && in->context) {
+                flb_info("[input] pausing %s", flb_input_name(in));
                 in->p->cb_pause(in->context, in->config);
             }
             paused++;
@@ -714,6 +828,10 @@ int flb_input_collector_pause(int coll_id, struct flb_input_instance *in)
         return -1;
     }
 
+    if (coll->running == FLB_FALSE) {
+        return 0;
+    }
+
     config = in->config;
     if (coll->type == FLB_COLLECT_TIME) {
         /*
@@ -806,6 +924,7 @@ int flb_input_set_collector_socket(struct flb_input_instance *in,
         return -1;
     }
 
+    collector->id          = collector_id(in);
     collector->type        = FLB_COLLECT_FD_SERVER;
     collector->cb_collect  = cb_new_connection;
     collector->fd_event    = fd;
