@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -45,31 +45,43 @@ ssize_t flb_input_chunk_get_size(struct flb_input_chunk *ic)
 
 int flb_input_chunk_write(void *data, const char *buf, size_t len)
 {
+    int ret;
     struct flb_input_chunk *ic;
 
     ic = (struct flb_input_chunk *) data;
 
-    return cio_chunk_write(ic->chunk, buf, len);
+    ret = cio_chunk_write(ic->chunk, buf, len);
+#ifdef FLB_HAVE_METRICS
+    if (ret == CIO_OK) {
+        ic->added_records = flb_mp_count(buf, len);
+        ic->total_records += ic->added_records;
+    }
+#endif
+
+    return ret;
 }
 
 int flb_input_chunk_write_at(void *data, off_t offset,
                              const char *buf, size_t len)
 {
+    int ret;
     struct flb_input_chunk *ic;
 
     ic = (struct flb_input_chunk *) data;
 
-    return cio_chunk_write_at(ic->chunk, offset, buf, len);
+    ret = cio_chunk_write_at(ic->chunk, offset, buf, len);
+    return ret;
 }
 
 /* Create an input chunk using a Chunk I/O */
 struct flb_input_chunk *flb_input_chunk_map(struct flb_input_instance *in,
                                             void *chunk)
 {
+#ifdef FLB_HAVE_METRICS
     int ret;
-    int records;
     char *buf_data;
     size_t buf_size;
+#endif
     struct flb_input_chunk *ic;
 
     /* Create context for the input instance */
@@ -93,9 +105,9 @@ struct flb_input_chunk *flb_input_chunk_map(struct flb_input_instance *in,
         return ic;
     }
 
-    records = flb_mp_count(buf_data, buf_size);
-    if (records > 0) {
-        flb_metrics_sum(FLB_METRIC_N_RECORDS, records, in->metrics);
+    ic->total_records = flb_mp_count(buf_data, buf_size);
+    if (ic->total_records > 0) {
+        flb_metrics_sum(FLB_METRIC_N_RECORDS, ic->total_records, in->metrics);
         flb_metrics_sum(FLB_METRIC_N_BYTES, buf_size, in->metrics);
     }
 #endif
@@ -124,8 +136,7 @@ struct flb_input_chunk *flb_input_chunk_create(struct flb_input_instance *in,
                            CIO_OPEN, FLB_INPUT_CHUNK_SIZE, &err);
     if (!chunk) {
         flb_error("[input chunk] could not create chunk file: %s:%s",
-                  storage->stream, name);
-        flb_sds_destroy(name);
+                  storage->stream->name, name);
         return NULL;
     }
 
@@ -153,7 +164,6 @@ struct flb_input_chunk *flb_input_chunk_create(struct flb_input_instance *in,
     ret = cio_meta_write(chunk, (char *) tag, tag_len);
     if (ret == -1) {
         flb_error("[input chunk] could not write metadata");
-        flb_sds_destroy(name);
         cio_chunk_close(chunk, CIO_TRUE);
         return NULL;
     }
@@ -170,6 +180,9 @@ struct flb_input_chunk *flb_input_chunk_create(struct flb_input_instance *in,
     ic->fs_backlog = FLB_FALSE;
     ic->in = in;
     ic->stream_off = 0;
+#ifdef FLB_HAVE_METRICS
+    ic->total_records = 0;
+#endif
     msgpack_packer_init(&ic->mp_pck, ic, flb_input_chunk_write);
     mk_list_add(&ic->_head, &in->chunks);
 
@@ -197,7 +210,7 @@ static struct flb_input_chunk *input_chunk_get(const char *tag, int tag_len,
     struct flb_input_chunk *ic = NULL;
 
     /* Try to find a current chunk context to append the data */
-    mk_list_foreach(head, &in->chunks) {
+    mk_list_foreach_r(head, &in->chunks) {
         ic = mk_list_entry(head, struct flb_input_chunk, _head);
         if (ic->busy == FLB_TRUE || cio_chunk_is_locked(ic->chunk)) {
             ic = NULL;
@@ -295,7 +308,7 @@ size_t flb_input_chunk_set_limits(struct flb_input_instance *in)
         in->mem_buf_status = FLB_INPUT_RUNNING;
         if (in->p->cb_resume) {
             in->p->cb_resume(in->context, in->config);
-            flb_debug("[input] %s resume (mem buf overlimit)",
+            flb_info("[input] %s resume (mem buf overlimit)",
                       in->name);
         }
     }
@@ -310,7 +323,7 @@ size_t flb_input_chunk_set_limits(struct flb_input_instance *in)
 static inline int flb_input_chunk_protect(struct flb_input_instance *i)
 {
     if (flb_input_chunk_is_overlimit(i) == FLB_TRUE) {
-        flb_debug("[input] %s paused (mem buf overlimit)",
+        flb_warn("[input] %s paused (mem buf overlimit)",
                  i->name);
         if (!flb_input_buf_paused(i)) {
             if (i->p->cb_pause) {
@@ -391,14 +404,11 @@ int flb_input_chunk_append_raw(struct flb_input_instance *in,
 {
     int ret;
     int set_down = FLB_FALSE;
+    int min;
     size_t size;
     struct flb_input_chunk *ic;
     struct flb_storage_input *si;
 
-#ifdef FLB_HAVE_METRICS
-    int records;
-#endif
-
     /* Check if the input plugin has been paused */
     if (flb_input_buf_paused(in) == FLB_TRUE) {
         flb_debug("[input chunk] %s is paused, cannot append records",
@@ -453,9 +463,8 @@ int flb_input_chunk_append_raw(struct flb_input_instance *in,
 
     /* Update 'input' metrics */
 #ifdef FLB_HAVE_METRICS
-    records = flb_mp_count(buf, buf_size);
-    if (records > 0) {
-        flb_metrics_sum(FLB_METRIC_N_RECORDS, records, in->metrics);
+    if (ic->total_records > 0) {
+        flb_metrics_sum(FLB_METRIC_N_RECORDS, ic->added_records, in->metrics);
         flb_metrics_sum(FLB_METRIC_N_BYTES, buf_size, in->metrics);
     }
 #endif
@@ -469,7 +478,7 @@ int flb_input_chunk_append_raw(struct flb_input_instance *in,
     size = cio_chunk_get_content_size(ic->chunk);
 
     /* Lock buffers where size > 2MB */
-    if (size > 2048000) {
+    if (size > FLB_INPUT_CHUNK_FS_MAX_SIZE) {
         cio_chunk_lock(ic->chunk);
     }
 
@@ -525,7 +534,20 @@ int flb_input_chunk_append_raw(struct flb_input_instance *in,
     if (flb_input_chunk_is_overlimit(in) == FLB_TRUE &&
         si->type == CIO_STORE_FS) {
         if (cio_chunk_is_up(ic->chunk) == CIO_TRUE) {
-            cio_chunk_down(ic->chunk);
+            /*
+             * If we are already over limit, a sub-sequent data ingestion
+             * might need a Chunk to write data in. As an optimization we
+             * will put this Chunk down ONLY IF it has less than 1% of
+             * it capacity as available space, otherwise keep it 'up' so
+             * it available space can be used.
+             */
+            size = cio_chunk_get_content_size(ic->chunk);
+
+            /* Do we have less than 1% available ? */
+            min = (FLB_INPUT_CHUNK_FS_MAX_SIZE * 0.01);
+            if (FLB_INPUT_CHUNK_FS_MAX_SIZE - size < min) {
+                cio_chunk_down(ic->chunk);
+            }
         }
         return 0;
     }
@@ -596,6 +618,8 @@ int flb_input_chunk_get_tag(struct flb_input_chunk *ic,
 
     ret = cio_meta_read(ic->chunk, &buf, &len);
     if (ret == -1) {
+        *tag_len = -1;
+        *tag_buf = NULL;
         return -1;
     }
 
