@@ -44,7 +44,11 @@ static inline int consume_byte(flb_pipefd_t fd)
 
     /* We need to consume the byte */
     ret = flb_pipe_r(fd, &val, sizeof(val));
+#ifdef __APPLE__
+    if (ret < 0) {
+#else
     if (ret <= 0) {
+#endif
         flb_errno();
         return -1;
     }
