@@ -86,7 +86,7 @@ static struct request *http_request_create(char *request)
     char *start;
     char *end;
     char *p;
-    char *br;
+    char *br = NULL;
     flb_sds_t tmp;
     flb_sds_t key;
     flb_sds_t val;
